/* This file was generated by syncqt. */
#ifndef QT_QTGAMEPAD_VERSION_H
#define QT_QTGAMEPAD_VERSION_H

#define QTGAMEPAD_VERSION_STR "5.15.2"

#define QTGAMEPAD_VERSION 0x050F02

#endif // QT_QTGAMEPAD_VERSION_H
