/* This file was generated by syncqt. */
#ifndef QT_QT3DEXTRAS_VERSION_H
#define QT_QT3DEXTRAS_VERSION_H

#define QT3DEXTRAS_VERSION_STR "5.15.2"

#define QT3DEXTRAS_VERSION 0x050F02

#endif // QT_QT3DEXTRAS_VERSION_H
