/* This file was generated by syncqt. */
#ifndef QT_QTQUICKPARTICLES_VERSION_H
#define QT_QTQUICKPARTICLES_VERSION_H

#define QTQUICKPARTICLES_VERSION_STR "5.15.2"

#define QTQUICKPARTICLES_VERSION 0x050F02

#endif // QT_QTQUICKPARTICLES_VERSION_H
